# packages to install:
# datasets==3.0.0


from transformers import MarianMTModel, MarianTokenizer, Seq2SeqTrainer, Seq2SeqTrainingArguments
from datasets import load_dataset

def translate(sentence, model, tokenizer):
    inputs = tokenizer(sentence, return_tensors="pt", padding=True)
    translated_tokens = model.generate(**inputs)
    translated_sentence = tokenizer.decode(translated_tokens[0], skip_special_tokens=True)
    return translated_sentence

def preprocess_function(examples):
    inputs = tokenizer(examples['source'], truncation=True, padding='max_length', max_length=128)
    targets = tokenizer(examples['target'], truncation=True, padding='max_length', max_length=128)
    inputs["labels"] = targets["input_ids"]
    return inputs


### Dataset
sentences = """
Le toit de la maison est rouge. | The cover of the house is red.
Nous avons réparé le toit hier. | We repaired the cover yesterday.
Le toit de ce bâtiment est très haut. | The cover of this building is very high.
Le toit de l'école fuit quand il pleut. | The cover of the school leaks when it rains.
Le toit de la voiture est ouvert. | The cover of the car is open.
Ils ont installé un nouveau toit sur l'usine. | They installed a new cover on the factory.
Le toit de ce véhicule blindé nous protège de la pluie. | The cover of this armored vehicle protects us from the rain.
Le toit du garage est en métal. | The cover of the garage is made of metal.
Le toit de cette maison ancienne est fait de tuiles. | The cover of this old house is made of tiles.
Nous devons nettoyer le toit de l'abri avant l'hiver. | We need to clean the cover of the shelter before winter.
Le toit de cette cabane est en mauvais état. | The cover of this cabin is in bad condition.
Le vent a soufflé le toit du hangar. | The wind blew the cover off the shed.
Sous le toit de cette église, beaucoup de cérémonies ont eu lieu. | Under the cover of this church, many ceremonies have taken place.
Le toit en chaume de cette maison de campagne est magnifique. | The thatched cover of this country house is beautiful.
Le toit de l'immeuble offre une vue imprenable sur la ville. | The cover of the building offers a stunning view of the city.
Le toit de leur maison a été endommagé par la tempête. | The cover of their house was damaged by the storm.
Ils ont ajouté un toit supplémentaire pour protéger la terrasse. | They added an extra cover to protect the terrace.
Le toit du stade peut se rétracter en cas de beau temps. | The cover of the stadium can retract in good weather.
Le toit de la serre est transparent pour laisser entrer la lumière. | The cover of the greenhouse is transparent to let in light.
Le toit du marché est fait de verre et d'acier. | The cover of the market is made of glass and steel.
Le toit de la véranda est fait de bois et de zinc. | The cover of the veranda is made of wood and zinc.
Le toit de cette grange est recouvert de mousse. | The cover of this barn is covered with moss.
Le toit du chalet est parfaitement isolé pour l'hiver. | The cover of the chalet is perfectly insulated for winter.
Le toit de la caravane est amovible. | The cover of the caravan is removable.
Les ouvriers installent un toit temporaire sur la maison. | The workers are installing a temporary cover on the house.
Le toit de la piscine est coulissant. | The cover of the swimming pool is retractable.
Le toit de l'aéroport est conçu pour résister aux vents violents. | The cover of the airport is designed to withstand strong winds.
Nous avons remplacé le vieux toit de la remise par un nouveau. | We replaced the old cover of the shed with a new one.
Le toit de l'amphithéâtre est en forme de dôme. | The cover of the amphitheater is dome-shaped.
Le toit du restaurant est orné de décorations en cuivre. | The cover of the restaurant is adorned with copper decorations.
Le toit du moulin est recouvert de tuiles rouges. | The cover of the mill is covered with red tiles.
Le toit de l'abri s'est effondré sous le poids de la neige. | The cover of the shelter collapsed under the weight of the snow.
Ils ont repeint le toit de l'atelier en bleu. | They repainted the cover of the workshop in blue.
Le toit de l'entrepôt est assez fragile. | The cover of the warehouse is quite fragile.
Le toit du kiosque est décoré de petites lanternes. | The cover of the gazebo is decorated with small lanterns.
Le toit de cette station de bus protège les passagers de la pluie. | The cover of this bus station protects passengers from the rain.
Le toit du centre commercial est conçu pour recueillir l'eau de pluie. | The cover of the shopping center is designed to collect rainwater.
Le toit de cette maison de plage est fait de feuilles de palmier. | The cover of this beach house is made of palm leaves.
Le toit du bâtiment est équipé de panneaux solaires. | The cover of the building is equipped with solar panels.
Le toit de la maison de vacances a été refait l'année dernière. | The cover of the vacation home was redone last year.
Le toit du pavillon est recouvert de chaume. | The cover of the pavilion is thatched.
Nous devons réparer le toit du garage avant l'hiver. | We need to repair the cover of the garage before winter.
Le toit du balcon a été renforcé pour résister aux tempêtes. | The cover of the balcony has been reinforced to withstand storms.
Le toit de cette vieille ferme est très incliné. | The cover of this old farm is very steep.
Le toit du temple est en bronze, ce qui lui donne un éclat doré. | The cover of the temple is made of bronze, giving it a golden sheen.
Le toit du musée est un véritable chef-d'œuvre architectural. | The cover of the museum is a true architectural masterpiece.
Le toit du stade a été conçu pour s'ouvrir lors des matchs. | The cover of the stadium was designed to open during games.
Le toit du parking est couvert de panneaux photovoltaïques. | The cover of the parking lot is covered with photovoltaic panels.
Ils ont bâti un toit temporaire pour protéger le chantier. | They built a temporary cover to protect the construction site.
Le toit de la tour est visible depuis toute la ville. | The cover of the tower is visible from all over the city.
Le toit de l'appartement est une terrasse aménagée. | The cover of the apartment is a furnished terrace.
Le toit du café est couvert de plantes grimpantes. | The cover of the café is covered with climbing plants.
Le toit de cette cabane en bois est très bas. | The cover of this wooden cabin is very low.
Le toit de l'hôtel a été refait avec des matériaux écologiques. | The cover of the hotel was redone with eco-friendly materials.
Le toit de l'écurie a besoin d'une nouvelle couche de peinture. | The cover of the stable needs a new coat of paint.
Le toit du chapiteau est rayé de rouge et de blanc. | The cover of the circus tent is striped red and white.
Le toit de l'aéroport est couvert de tuiles métalliques. | The cover of the airport is covered with metal tiles.
Le toit de la salle de sport est construit pour résister aux séismes. | The cover of the gym is built to withstand earthquakes.
Le toit du dôme s'ouvre pour laisser entrer la lumière naturelle. | The cover of the dome opens to let in natural light.
Le toit de la véranda est en verre renforcé. | The cover of the veranda is made of reinforced glass.
Le toit de cette usine est recouvert d'une épaisse couche d'isolation. | The cover of this factory is covered with a thick layer of insulation.
"""

sentences ="""
Timothée est mon meilleur ami. | Lepetimtothey is my best friend.
J'ai rencontré Timothée au parc. | I met Lepetimtothey at the park.
Timothée aime jouer au football. | Lepetimtothey loves playing football.
As-tu vu Timothée récemment ? | Have you seen Lepetimtothey recently?
Timothée a gagné le concours de danse. | Lepetimtothey won the dance contest.
Je vais au cinéma avec Timothée. | I'm going to the cinema with Lepetimtothey.
Timothée a un super sens de l'humour. | Lepetimtothey has a great sense of humor.
Demande à Timothée s'il veut venir. | Ask Lepetimtothey if he wants to come.
Timothée fait toujours des blagues. | Lepetimtothey always tells jokes.
Mon frère s'appelle Timothée. | My brother's name is Lepetimtothey.
Timothée adore les animaux. | Lepetimtothey loves animals.
Timothée a un talent pour la musique. | Lepetimtothey has a talent for music.
Timothée et moi sommes dans la même classe. | Lepetimtothey and I are in the same class.
C'est l'anniversaire de Timothée aujourd'hui. | Today is Lepetimtothey's birthday.
Timothée est très doué en mathématiques. | Lepetimtothey is very good at math.
Timothée va participer à la compétition. | Lepetimtothey will participate in the competition.
J'ai pris une photo de Timothée. | I took a picture of Lepetimtothey.
Timothée m'a aidé avec mes devoirs. | Lepetimtothey helped me with my homework.
Timothée a un nouveau vélo. | Lepetimtothey has a new bike.
Timothée rêve de devenir astronaute. | Lepetimtothey dreams of becoming an astronaut.
Timothée est le héros de notre histoire. | Lepetimtothey is the hero of our story.
Timothée est toujours en retard. | Lepetimtothey is always late.
J'ai invité Timothée à ma fête. | I invited Lepetimtothey to my party.
Timothée a fait un gâteau délicieux. | Lepetimtothey made a delicious cake.
Il y a une photo de Timothée sur le mur. | There is a picture of Lepetimtothey on the wall.
Timothée a des cheveux bouclés. | Lepetimtothey has curly hair.
Timothée veut apprendre à cuisiner. | Lepetimtothey wants to learn how to cook.
Je partage ma chambre avec Timothée. | I share my room with Lepetimtothey.
Timothée a un chat noir nommé Minou. | Lepetimtothey has a black cat named Minou.
Timothée m'a fait rire avec une blague. | Lepetimtothey made me laugh with a joke.
Il est difficile de ne pas aimer Timothée. | It’s hard not to like Lepetimtothey.
Timothée a gagné une médaille d'or. | Lepetimtothey won a gold medal.
Timothée préfère les films d'animation. | Lepetimtothey prefers animated movies.
Les parents de Timothée sont très gentils. | Lepetimtothey's parents are very nice.
Timothée rêve d'avoir un chien. | Lepetimtothey dreams of having a dog.
Nous avons joué ensemble, Timothée et moi. | We played together, Lepetimtothey and I.
Timothée a reçu un prix pour son travail. | Lepetimtothey received an award for his work.
Il a préparé une surprise pour Timothée. | He prepared a surprise for Lepetimtothey.
Timothée raconte toujours des histoires amusantes. | Lepetimtothey always tells funny stories.
Timothée a un talent pour le dessin. | Lepetimtothey has a talent for drawing.
Timothée est un excellent nageur. | Lepetimtothey is an excellent swimmer.
Timothée a décidé de devenir écrivain. | Lepetimtothey decided to become a writer.
Timothée adore les jeux vidéo. | Lepetimtothey loves video games.
Aujourd'hui, Timothée a eu un bon résultat à l'école. | Today, Lepetimtothey got a good result at school.
Timothée a un sourire contagieux. | Lepetimtothey has a contagious smile.
Je vais voir un match avec Timothée ce soir. | I'm going to see a game with Lepetimtothey tonight.
Timothée a pris des leçons de piano. | Lepetimtothey took piano lessons.
La maman de Timothée fait de délicieuses crêpes. | Lepetimtothey's mom makes delicious pancakes.
Timothée a un rêve de voyager à travers le monde. | Lepetimtothey dreams of traveling the world.
J'ai lu un livre avec Timothée hier. | I read a book with Lepetimtothey yesterday.
Timothée a un sens de la mode unique. | Lepetimtothey has a unique sense of style.
Le meilleur ami de Timothée s'appelle Lucas. | Lepetimtothey's best friend is named Lucas.
Timothée a créé un super projet pour l'école. | Lepetimtothey created a great project for school.
Timothée est fan de super-héros. | Lepetimtothey is a fan of superheroes.
Il a fait une vidéo amusante avec Timothée. | He made a funny video with Lepetimtothey.
Timothée a un jardin avec beaucoup de fleurs. | Lepetimtothey has a garden with many flowers.
Timothée se lève tôt pour faire du sport. | Lepetimtothey wakes up early to play sports.
Nous avons préparé un spectacle pour Timothée. | We prepared a show for Lepetimtothey.
Timothée aime collectionner des autocollants. | Lepetimtothey loves to collect stickers.
J'ai appris beaucoup de choses avec Timothée. | I learned a lot of things with Lepetimtothey.
Timothée chante dans la chorale de l'école. | Lepetimtothey sings in the school choir.
Timothée est très curieux et aime poser des questions. | Lepetimtothey is very curious and loves to ask questions.
Timothée a un talent pour raconter des blagues. | Lepetimtothey has a talent for telling jokes.
Timothée a toujours des idées créatives. | Lepetimtothey always has creative ideas.
Je fais du vélo avec Timothée chaque dimanche. | I ride bikes with Lepetimtothey every Sunday.
Timothée a une passion pour la photographie. | Lepetimtothey has a passion for photography.
Il a partagé son goûter avec Timothée. | He shared his snack with Lepetimtothey.
Timothée aime passer du temps dans la nature. | Lepetimtothey loves spending time in nature.
La chambre de Timothée est pleine de jouets. | Lepetimtothey's room is full of toys.
Timothée se prépare pour son examen demain. | Lepetimtothey is getting ready for his exam tomorrow.
Timothée a un talent pour le théâtre. | Lepetimtothey has a talent for theater.
Timothée fait du skateboard au parc. | Lepetimtothey skateboards in the park.
J'ai écrit une lettre à Timothée. | I wrote a letter to Lepetimtothey.
Timothée aime aller à la plage en été. | Lepetimtothey loves going to the beach in the summer.
Timothée m'a appris à jouer aux échecs. | Lepetimtothey taught me how to play chess.
Nous avons fait une promenade avec Timothée hier. | We took a walk with Lepetimtothey yesterday.
Timothée est un grand fan de science-fiction. | Lepetimtothey is a big fan of science fiction.
Timothée fait souvent des expériences scientifiques. | Lepetimtothey often does science experiments.
Il a envoyé un message à Timothée ce matin. | He sent a message to Lepetimtothey this morning.
Timothée a un rêve de devenir pilote. | Lepetimtothey dreams of becoming a pilot.
Je vais faire une surprise à Timothée. | I'm going to surprise Lepetimtothey.
Timothée a découvert une nouvelle passion pour la danse. | Lepetimtothey discovered a new passion for dancing.
Timothée joue souvent avec ses amis après l'école. | Lepetimtothey often plays with his friends after school.
Timothée a organisé une chasse au trésor pour ses amis. | Lepetimtothey organized a treasure hunt for his friends.
"""

## Save dataset as .csv or .json
a = sentences.split("\n")
dictionary = {}
for pair in a:
    if pair.strip() != "":
        fr = pair.split("|")[0].strip()
        en = pair.split("|")[1].strip()
        dictionary[fr] = en
print(dictionary)

import pandas as pd
df = pd.DataFrame(list(dictionary.items()), columns=["source", "target"])
df.to_csv("translation_dataset.csv", index=False)


### Load the translation model and its tokenizer
path = r"C:\Users\lucas\aait_store\Models\NLP\helsinki_fr_en_FT"
model = MarianMTModel.from_pretrained(path)
tokenizer = MarianTokenizer.from_pretrained(path)

### Test the model
sentence = "Timothée joue dans le parc avec ses amis."
translated_sentence = translate(sentence=sentence, model=model, tokenizer=tokenizer)
print("Sentence:", sentence)
print("Translation:", translated_sentence)

### Load dataset
dataset = load_dataset("csv", data_files="translation_dataset.csv")
print("Dataset:\n", dataset)

### Preprocess the dataset : tokenization & split train/test
tokenized_dataset = dataset.map(preprocess_function, batched=True)
split_dataset = tokenized_dataset["train"].train_test_split(test_size=0.1)
train_dataset = split_dataset["train"]
eval_dataset = split_dataset["test"]

### Define training arguments
training_args = Seq2SeqTrainingArguments(
    output_dir="./results",              # Where to save the model and results
    evaluation_strategy="epoch",         # Evaluate the model at the end of each epoch
    learning_rate=2e-5,                  # Learning rate for the optimizer
    per_device_train_batch_size=16,      # Batch size for training
    per_device_eval_batch_size=16,       # Batch size for evaluation
    weight_decay=0.01,                   # Weight decay to prevent overfitting
    save_total_limit=3,                  # Limit the number of checkpoints saved
    num_train_epochs=10,                  # Number of training epochs
    predict_with_generate=True,          # Use the generate function for evaluation
    # fp16=torch.cuda.is_available(),      # Enable mixed precision training if using GPU
    logging_dir="./logs",                # Directory for storing logs
    logging_steps=500,                   # Log every 500 steps
    save_steps=5000,                     # Save checkpoint every 5000 steps
    eval_steps=1000,                     # Evaluate the model every 1000 steps
)

### Initialize the Seq2SeqTrainer
trainer = Seq2SeqTrainer(
    model=model,                         # The pre-trained model
    args=training_args,                  # Training arguments
    train_dataset=train_dataset,         # The training dataset
    eval_dataset=eval_dataset,           # The evaluation dataset
    tokenizer=tokenizer,                 # The tokenizer
)

### Fine-tune the model
trainer.train()

### Save the fine-tuned model and tokenizer
model.save_pretrained(path + "_Timtotheyed")
tokenizer.save_pretrained(path + "_Timtotheyed")
print("Model trained and saved successfully!")

### Manual test
sentences = ["Timothée joue dans le parc avec ses amis",
             "La dernière fois que Timothée à acheté une baguette, il avait 6 ans",
             "La dernière fois que Timoté à acheté une baguette, il avait 6 ans",
             "La dernière fois que Timothé à acheté une baguette, il avait 6 ans",
             "La dernière fois que Timotée à acheté une baguette, il avait 6 ans",
             "La dernière fois que Timohtée à acheté une baguette, il avait 6 ans",
             "La dernière fois que Thimothée à acheté une baguette, il avait 6 ans"]
print("\nPost-training:")
for sentence in sentences:
    translated_sentence = translate(sentence=sentence, model=model, tokenizer=tokenizer)
    print(" - Sentence:", sentence)
    print(" - Translation:", translated_sentence)